--
-- DependentAnimation
-- Specialization for animations dependent on movingTools
--
-- @author  Stefan Maurus	
-- @date  2015-03-24

-- Copyright © Stefan Maurus, www.stefanmaurus.de

DependentAnimation = {};

function DependentAnimation.prerequisitesPresent(specializations)
    return true;
end;

function DependentAnimation:load(xmlFile)
	local i = 0;
	self.dependentAnimations = {};
	while true do
		local key = string.format("vehicle.dependentAnimations.animation(%d)", i);
		if not hasXMLProperty(xmlFile, key) then
			break;
		end;
		
		local movingToolIndex = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#movingToolIndex"));	
		
		local invert = Utils.getNoNil(getXMLBool(xmlFile, key.."#invert"), false);
		
		local y = 0;
		local parts = {};
		while true do
			local key2 = string.format(key..".part(%d)", y);
			if not hasXMLProperty(xmlFile, key2) then
				break;
			end;
			
			local index = Utils.indexToObject(self.components, getXMLString(xmlFile, key2.."#index"));	
			
			local animCurve = AnimCurve:new(linearInterpolatorTransRotScale);
		
			local yy = 0;
			while true do
				local key3 = string.format(key2..".keyframe(%d)", yy);
				if not hasXMLProperty(xmlFile, key3) then
					break;
				end;
				
				local t = getXMLFloat(xmlFile, key3.."#time");	
				local xST, yST, zST = Utils.getVectorFromString(getXMLString(xmlFile, key3.."#translation"));
				if xST == nil or yST == nil or zST == nil then
					xST, yST, zST = getTranslation(index);
				end;
				
				local xSR, ySR, zSR = Utils.getVectorFromString(getXMLString(xmlFile, key3.."#rotation"));
				if xSR == nil or ySR == nil or zSR == nil then
					xSR, ySR, zSR = getRotation(index);
				else
					xSR = math.rad(xSR);
					ySR = math.rad(ySR);
					zSR = math.rad(zSR);			
				end;

				local xSS, ySS, zSS = Utils.getVectorFromString(getXMLString(xmlFile, key3.."#scale"));
				if xSS == nil or ySS == nil or zSS == nil then
					xSS, ySS, zSS = getScale(index);
				end;			
				
				animCurve:addKeyframe({x=xST, y=yST, z=zST, rx=xSR, ry=ySR, rz=zSR, sx=xSS, sy=ySS, sz=zSS, time=t});
				
				yy = yy + 1;
			end;	
			
			
			table.insert(parts, {index=index, animCurve=animCurve});
			y = y + 1;
		end;			

		table.insert(self.dependentAnimations, {movingToolIndex=movingToolIndex, parts=parts, invert=invert});
		i = i + 1;
	end;	
end;

function DependentAnimation:delete()
end;

function DependentAnimation:mouseEvent(posX, posY, isDown, isUp, button)
end;

function DependentAnimation:keyEvent(unicode, sym, modifier, isDown)
end;

function DependentAnimation:readStream(streamId, connection)
end;

function DependentAnimation:writeStream(streamId, connection) 	
end;

function DependentAnimation:update(dt)	
end;

function DependentAnimation:updateTick(dt)
	for _, animation in pairs(self.dependentAnimations) do
		if self.nodesToMovingTools[animation.movingToolIndex].lastRotSpeed ~= 0 then
			local x, y, z = getRotation(self.nodesToMovingTools[animation.movingToolIndex].node);
			local t = 1-math.abs(math.max(math.min((x-self.nodesToMovingTools[animation.movingToolIndex].rotMin)/(self.nodesToMovingTools[animation.movingToolIndex].rotMax-self.nodesToMovingTools[animation.movingToolIndex].rotMin), 1), 0));
			if animation.invert then
				t = math.abs(1-t);
			end;
			for _, part in pairs(animation.parts) do
				local x,y,z, rx,ry,rz, sx,sy,sz = part.animCurve:get(t);	
				setTranslation(part.index, x,y,z);
				setRotation(part.index, rx, ry, rz);
				setScale(part.index, sx,sy,sz);		
			end;
		end;
		if self.nodesToMovingTools[animation.movingToolIndex].lastTransSpeed ~= 0 then
			local x, y, z = getRotation(self.nodesToMovingTools[animation.movingToolIndex].node);
			local t = 1-math.abs(math.max(math.min((x-self.nodesToMovingTools[animation.movingToolIndex].transMin)/(self.nodesToMovingTools[animation.movingToolIndex].transMax-self.nodesToMovingTools[animation.movingToolIndex].transMin), 1), 0));
			if animation.invert then
				t = math.abs(1-t);
			end;
			for _, part in pairs(animation.parts) do
				local x,y,z, rx,ry,rz, sx,sy,sz = part.animCurve:get(t);	
				setTranslation(part.index, x,y,z);
				setRotation(part.index, rx, ry, rz);
				setScale(part.index, sx,sy,sz);		
			end;
		end;	
	end;
end;

function DependentAnimation:draw()
end;